@@ -16,7 +16,6 @@
 
 from __future__ import generators
 
-import calendar
 import tempfile
 import gobject
 import shutil
@@ -136,19 +135,12 @@ def _lookup_cvs_files(dirs, files):
                 if plus >= 0:
                     state = tree.STATE_CONFLICT
                 else:
-                    try:
-                        cotime = calendar.timegm( time.strptime(date) )
-                    except ValueError, e:
-                        if not date.startswith("Result of merge"):
-                            #print "Unable to parse date '%s' in '%s/CVS/Entries' (%s)" % (date, directory, e)
-                            pass
-                        cotime = 0
                     try:
                         mtime = os.stat(path).st_mtime
                     except OSError:
                         state = tree.STATE_MISSING
                     else:
-                        if mtime==cotime:
+                        if time.asctime(time.gmtime(mtime))==date:
                             state = tree.STATE_NORMAL
                         else:
                             state = tree.STATE_MODIFIED
@@ -521,14 +513,22 @@ class CvsView(melddoc.MeldDoc, gnomeglade.Component):
     def on_button_filter_toggled(self, button):
         self.refresh()
 
-    def _get_selected_files(self):
-        ret = []
+    def _get_selected_treepaths(self):
+        sel = []
         def gather(model, path, iter):
-            ret.append( model.value_path(iter,0) )
+            sel.append( model.get_path(iter) )
+        s = self.treeview.get_selection()
+        s.selected_foreach(gather)
+        return sel
+
+    def _get_selected_files(self):
+        sel = []
+        def gather(model, path, iter):
+            sel.append( model.value_path(iter,0) )
         s = self.treeview.get_selection()
         s.selected_foreach(gather)
         # remove empty entries and remove trailing slashes
-        return map(lambda x: x[-1]!="/" and x or x[:-1], filter(lambda x: x!=None, ret))
+        return [ x[-1]!="/" and x or x[:-1] for x in sel if x != None ]
 
     def _command_iter(self, command, files, refresh):
         """Run 'command' on 'files'. Return a tuple of the directory the
@@ -537,14 +537,14 @@ class CvsView(melddoc.MeldDoc, gnomeglade.Component):
         msg = misc.shelljoin(command)
         yield "[%s] %s" % (self.label_text, msg.replace("\n", u"\u21b2") )
         if len(files) == 1 and os.path.isdir(files[0]):
-            workdir = files[0]
-            files = ["."]
+            workdir = os.path.dirname( files[0] )
+            files = [ os.path.basename( files[0] ) ]
         else:
             workdir = _commonprefix(files)
-        kill = len(workdir) and (len(workdir)+1) or 0
-        files = filter(lambda x: len(x), map(lambda x: x[kill:], files))
+            kill = len(workdir) and (len(workdir)+1) or 0
+            files = filter(lambda x: len(x), map(lambda x: x[kill:], files))
         r = None
-        self.consolestream.write( misc.shelljoin(command+files) + "(in %s)\n" % workdir)
+        self.consolestream.write( misc.shelljoin(command+files) + " (in %s)\n" % workdir)
         readfunc = misc.read_pipe_iter(command + files, self.consolestream, workdir=workdir).next
         try:
             while r == None:
@@ -743,5 +743,21 @@ class CvsView(melddoc.MeldDoc, gnomeglade.Component):
         item.show()
         menu.insert( item, 1 )
 
+    def next_diff(self, direction):
+        start_iter = self.model.get_iter( (self._get_selected_treepaths() or [(0,)])[-1] )
+
+        def goto_iter(it):
+            curpath = self.model.get_path(it)
+            for i in range(len(curpath)-1):
+                self.treeview.expand_row( curpath[:i+1], 0)
+            self.treeview.set_cursor(curpath)
+
+        search = {gtk.gdk.SCROLL_UP : self.model.inorder_search_up}.get(direction, self.model.inorder_search_down)
+        for it in search( start_iter ):
+            state = int(self.model.get_state( it, 0))
+            if state not in (tree.STATE_NORMAL, tree.STATE_EMPTY):
+                goto_iter(it)
+                return
+
 gobject.type_register(CvsView)
 
